'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.hook = hook;

var _method = require('./method.js');

var _publication = require('./publication.js');

var _roles = require('./roles.js');

var hookMethods = function hookMethods() {
  console.log('hookMethods');

  var oldMethods = Meteor.methods;

  var newMethods = function newMethods() {
    var methods = arguments.length <= 0 ? undefined : arguments[0];

    var wrapMethods = {};

    var _loop = function _loop(m) {
      if (methods.hasOwnProperty(m)) {
        wrapMethods[m] = function wrapMethod() {
          var userId = Meteor.userId();

          if (!(0, _method.doesMethodExist)(m)) {
            if (Meteor.isDevelopment) {
              console.warn('Meteor ABAC: method ' + m + ' has not been checked');
            }
          } else {
            if (!(0, _method.isMethodPublished)(m) && !(0, _roles.canUserExecuteMethod)(userId, m)) {
              if (Meteor.isDevelopment) {
                throw new Meteor.Error(403, 'Do not have permission to execute ' + m);
              }
              throw new Meteor.Error(403, 'Do not have permission');
            }
          }
          return methods[m].bind(this).apply(undefined, arguments);
        };
      }
    };

    for (var m in methods) {
      _loop(m);
    }

    return oldMethods(wrapMethods);
  };

  Meteor.methods = newMethods;
}; // import { Meteor } from 'meteor/meteor';

var hookPublish = function hookPublish() {
  console.log('hookPublish');
  var oldPublish = Meteor.publish;

  var newPublish = function newPublish(name, func) {
    var wrapFunc = function wrapFunc() {
      var userId = this.userId;

      if (!(0, _publication.doesPublicationExist)(name)) {
        if (Meteor.isDevelopment) {
          console.warn('Meteor ABAC: publication ' + name + ' has not been checked');
        }
      } else {
        if (!(0, _publication.isPublicationPublished)(name) && !(0, _roles.canUserSubscribePublication)(userId, name)) {
          if (Meteor.isDevelopment) {
            console.error('Do not have permission to subscribe ' + name);
            throw new Meteor.Error(403, 'Do not have permission to subscribe ' + name);
          }
          throw new Meteor.Error(403, 'Do not have permission');
        }
      }

      return func.bind(this).apply(undefined, arguments);
    };
    return oldPublish(name, wrapFunc);
  };

  Meteor.publish = newPublish;
};

function hook() {
  hookMethods();
  hookPublish();
}